-----------------------------------------------------------------------------------------------
-- Client Lua Script for ResourceCounter
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

-- v.1.04 
-- You can now disable or enable pause on mouseover with /rcpause command.
-- v.1.03 
-- fixed a bug when crafting a Technologist Tincture, addon was gives error and does not calculates total count of Tinctures.
-- v.1.02 
-- fixed a bug that caused settler items was not calculating.
-- v.1.01 
-- fixed a bug that caused items with more than 1 stack in inventory bag was not calculating.

require "Window"
require "Sound"
require "GameLib"
require "ChatSystemLib"

local ResourceCounter = {}

local knMaxEntryData = 4 -- Previously 3
local kfMaxItemTime = 7	-- item display time (seconds)
local kfTimeBetweenItems = 2 -- Previously .3			-- delay between items; also determines clearing time (seconds)
local knType_Invalid = 0
local knType_Item = 1
local nChannelSlash = nil

local karQualitySquareSprite =
{
	[Item.CodeEnumItemQuality.Inferior] 		= "CRB_Tooltips:sprTooltip_Header_Silver",
	[Item.CodeEnumItemQuality.Average] 			= "CRB_Tooltips:sprTooltip_Header_White",
	[Item.CodeEnumItemQuality.Good] 			= "CRB_Tooltips:sprTooltip_Header_Green",
	[Item.CodeEnumItemQuality.Excellent] 		= "CRB_Tooltips:sprTooltip_Header_Blue",
	[Item.CodeEnumItemQuality.Superb] 			= "CRB_Tooltips:sprTooltip_Header_Purple",
	[Item.CodeEnumItemQuality.Legendary] 		= "CRB_Tooltips:sprTooltip_Header_Orange",
	[Item.CodeEnumItemQuality.Artifact]		 	= "CRB_Tooltips:sprTooltip_Header_Pink",
}

local karEvalColors =
{
	[Item.CodeEnumItemQuality.Inferior] 		= "ItemQuality_Inferior",
	[Item.CodeEnumItemQuality.Average] 			= "ItemQuality_Average",
	[Item.CodeEnumItemQuality.Good] 			= "ItemQuality_Good",
	[Item.CodeEnumItemQuality.Excellent] 		= "ItemQuality_Excellent",
	[Item.CodeEnumItemQuality.Superb] 			= "ItemQuality_Superb",
	[Item.CodeEnumItemQuality.Legendary] 		= "ItemQuality_Legendary",
	[Item.CodeEnumItemQuality.Artifact]		 	= "ItemQuality_Artifact",
}

function ResourceCounter:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
	o.arEntries = {}
	o.tEntryData = {}
	o.tQueuedEntryData = {}
	o.fLastTimeAdded = 0
    return o
end

function ResourceCounter:Init()
    Apollo.RegisterAddon(self)
end

function ResourceCounter:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("LootStack.xml")
	self.xmlDoc:RegisterCallback("OnDocumentReady", self) 
	
	 
end

function ResourceCounter:OnItemRemoved(itemSold, nCount, eReason) -- called when an item is sold to a vendor
  local tQueuedData = self.tQueuedEntryData[1]
	--table.remove(self.tQueuedEntryData, 1)
	if tQueuedData == nil then
		return
	end
	if tQueuedData.stack >1 and itemSold:GetItemId() == tQueuedData.itemInstance:GetItemId() then
	tQueuedData.stack = 0
	end
	
end

  

function ResourceCounter:OnConfigure()
	if self.tSavedParams.pauseOnHover then
	self.bPause = false
	self.timerUpdate:Start()
	self.tSavedParams.pauseOnHover = false 
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "Resource Counter - Pause on mouseover is Disabled")
	else
	self.tSavedParams.pauseOnHover = true
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "Resource Counter - Pause on mouseover is Enabled")
	end
	
	ResourceCounter:OnSave(GameLib.CodeEnumAddonSaveLevel.Character)
	
end

function ResourceCounter:OnSave(eType)
   

	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then 
	 return nil
	end
	
	 if self.tSavedParams == nil then
       self.tSavedParams = {}
	   self.tSavedParams.pauseOnHover = true
    end 
	local mSavedParams = {}
	 mSavedParams.pauseOnHover = self.tSavedParams.pauseOnHover
	return mSavedParams
end
function ResourceCounter:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	 if self.tSavedParams == nil then
       self.tSavedParams = {}
	   self.tSavedParams.pauseOnHover = true
    end 
	self.tSavedParams.pauseOnHover = tSavedData.pauseOnHover
	
end

function ResourceCounter:OnDocumentReady()
	if self.xmlDoc == nil then
		return
	end
	Apollo.RegisterEventHandler("ItemRemoved", "OnItemRemoved", self)
	Apollo.RegisterEventHandler("ChannelUpdate_Loot",	"OnChannelUpdate_Loot", self)
	Apollo.RegisterSlashCommand("rcpause", "OnConfigure", self)
	
	
	
	self.timerUpdate = ApolloTimer.Create(0.1, true, "OnLootStackUpdate", self)

	self.timerCash = ApolloTimer.Create(5.0, false, "OnLootStack_CashTimer", self)

	self.wndLootStack = Apollo.LoadForm(self.xmlDoc, "LootStackForm", "FixedHudStratumHigh", self)
	self.xmlDoc = nil
	self.wndCashDisplay = self.wndLootStack:FindChild("LootFloaters:CashComplex:CashDisplay")
	self.wndCashComplex = self.wndLootStack:FindChild("LootFloaters:CashComplex")
	
	-- This will be updated the first time we go through LootStackUpdate
	self.bIsMoveable = nil

	for idx = 1, knMaxEntryData do
		self.arEntries[idx] = self.wndLootStack:FindChild("LootFloaters:LootedItem_"..idx)
	end

	self:UpdateDisplay()

	Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
	self:OnWindowManagementReady()
	
	
	 if self.tSavedParams == nil then
       self.tSavedParams = {}
	   self.tSavedParams.pauseOnHover = true
    end 
end

function ResourceCounter:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementRegister", {wnd = self.wndLootStack, strName = Apollo.GetString("HUDAlert_VacuumLoot"), nSaveVersion=2})
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndLootStack, strName = Apollo.GetString("HUDAlert_VacuumLoot"), nSaveVersion=2})
end

function ResourceCounter:OnGenerateTooltip(wndHandler, wndControl)
	if not wndControl or not wndControl:IsValid() or not wndControl:GetData() then
		return
	end
	
	itemCurr = wndHandler:GetData()
	
	wndControl:SetTooltipDoc(nil)
	if itemCurr ~= nil then
		Tooltip.GetItemTooltipForm(self, wndControl, itemCurr, {bPrimary = true, bSelling = false, itemCompare = itemCurr:GetEquippedItemForItemType()})
	end
end

function ResourceCounter:OnMouseEnter(wndHandler, wndControl)
    if self.tSavedParams.pauseOnHover then  
       self.bPause = true
	   self.timerUpdate:Stop()
	end
end

function ResourceCounter:OnMouseExit(wndHandler, wndControl)
    if self.tSavedParams.pauseOnHover then  
	   self.bPause = false
	   self.timerUpdate:Start()
	end
end

-----------------------------------------------------------------------------------------------
-- Channel Update
-----------------------------------------------------------------------------------------------

function ResourceCounter:OnChannelUpdate_Loot(eType, tEventArgs)
local stackz = 0
	if eType == GameLib.ChannelUpdateLootType.Currency and tEventArgs.monNew then

		local eCurrencyType = tEventArgs.monNew:GetMoneyType()
		if eCurrencyType ~= Money.CodeEnumCurrencyType.Credits then
			return
		end

		self.wndCashDisplay:SetAmount(self.wndCashDisplay:GetAmount() + tEventArgs.monNew:GetAmount())
		self.wndCashComplex:Invoke()

		self.timerCash:Stop()
		self.timerCash:Start()
		
	elseif eType == GameLib.ChannelUpdateLootType.Item and tEventArgs.itemNew then
       local found = false
	   local familyName = tEventArgs.itemNew:GetItemFamilyName()
	   local categoryName = tEventArgs.itemNew:GetItemCategoryName()
	   local typeName  = tEventArgs.itemNew:GetItemTypeName()
	   local family = tEventArgs.itemNew:GetItemFamily()
	   local category = tEventArgs.itemNew:GetItemCategory()
	   local itype  = tEventArgs.itemNew:GetItemType()
	  
		
		--[[
		Print(typeName)
		Print(String_GetWeaselString(itype))
		Print(familyName)
		 Print(String_GetWeaselString(family))
		Print(categoryName)
		Print(String_GetWeaselString(category))
		]]--
		
	--if category ==94 then --Junk
	--end
		--Crafting (Family)
		--Junk
		--Runecrafting(SetFocus,Sign of Fusion,Sign of Fire,Sign of Earth,Sign of Logic,Sign of Air,Sign of Life,Sign of Water)
		--Survivalist(Meat,Leather)
		--Farming (Herb)
		--PowerCore
	--[[	
	if family == 35 then --Token
	   if category ==178 then --WorldBossToken
	       for _, item in ipairs(GameLib.GetPlayerUnit():GetInventoryItems()) do
		        local itemz2 = tEventArgs.itemNew:GetItemId()
			    if itemz2 == item.itemInBag:GetItemId() then
	                stackz = stackz + item.itemInBag:GetStackCount()
				end
		   
            end
	    end
	else]]--
	if family == 29 then --Path
	
	   if category ==111 then --Settler
	      if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Settler Resources"] ~=nil then
		      for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Settler Resources"]) do
		           local itemz2 = tEventArgs.itemNew:GetItemId()
			       if itemz2 == item.itemMaterial:GetItemId() then
	                 stackz = item.nCount
					  found = true
					  break
	               end
	          end
		   end
	   end
		 
	   for _, item in ipairs(GameLib.GetPlayerUnit():GetInventoryItems()) do
		     local itemz2 = tEventArgs.itemNew:GetItemId()
			    if itemz2 == item.itemInBag:GetItemId() then
	                stackz = stackz + item.itemInBag:GetStackCount()
				end
		   
       end
	
	elseif family == 27  then --Crafting
	    if category ==110 then --Survivalist
	        if itype ==207 or itype ==274 or itype ==275 then --leather --pelt --bone
		     if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Leather and Pelts"] ~=nil then
	              for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Leather and Pelts"]) do
		              local itemz2 = tEventArgs.itemNew:GetItemId()
			            if itemz2 == item.itemMaterial:GetItemId() then
				           stackz = item.nCount
					       found = true
					
					       break
	                   end
	               end
		       end
	        elseif itype ==219  then --wood
		       if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Wood"] ~=nil then
		            for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Wood"]) do
		                local itemz2 = tEventArgs.itemNew:GetItemId()
			            if itemz2 == item.itemMaterial:GetItemId() then
				           stackz = item.nCount
					       found = true
					       break
	                   end
	                end
			   end
	       elseif itype ==208 or itype ==268 or itype ==266 or itype ==269 then --meat -- bug meat --fish --poultry
		
		        --Meat
		        if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Cooking"] ~=nil then
		            for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Cooking"]) do
		                local itemz2 = tEventArgs.itemNew:GetItemId()
			           if itemz2 == item.itemMaterial:GetItemId() then
	                        stackz = item.nCount
					        found = true
					        break
	                   end
	               end
		        end
	        elseif itype ==520 or itype ==519 or itype ==211 then --cloth -- weave -fiber
		        if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Cloth"] ~=nil then
		            for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Cloth"]) do
		               local itemz2 = tEventArgs.itemNew:GetItemId()
			            if itemz2 == item.itemMaterial:GetItemId() then
	                        stackz = item.nCount
					        found = true
					        break
	                    end
	                end
			    end
		    end
	       -- rune fragment 515
	    elseif category ==177 then --Runecrafting
	        if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Runecrafting"] ~=nil then
		        for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Runecrafting"]) do
		            local itemz2 = tEventArgs.itemNew:GetItemId()
			        if itemz2 == item.itemMaterial:GetItemId() then
	                  stackz = item.nCount
					  found = true
					   -- Print(tostring(item.itemMaterial:isInstance()).."a")
					  break
	                end
	            end
		    end
		
           -- 527 itype power core
	    elseif category ==143 then --Power Core
	        if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Power Cores"] ~=nil then
		        for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Power Cores"]) do
		            local itemz2 = tEventArgs.itemNew:GetItemId()
			        if itemz2 == item.itemMaterial:GetItemId() then
	                    stackz = item.nCount
					    found = true
					    break
	                end
	            end
		    end
		
           --Herb 198 itype
	       --Seeds 213 itype
	    elseif category ==104 then --Farming
	        if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Herbs and Produce"] ~=nil then
		        for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Herbs and Produce"]) do
		            local itemz2 = tEventArgs.itemNew:GetItemId()
			        if itemz2 == item.itemMaterial:GetItemId() then
	                    stackz = item.nCount
					    found = true
					    break
	                end
			    end
	        end
		
        elseif category ==103 then --Mining
	        -- Ore 197 --Gem 270
		    if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Ore & Gems"] ~=nil then
		        for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Ore & Gems"]) do
		            local itemz2 = tEventArgs.itemNew:GetItemId()
			        if itemz2 == item.itemMaterial:GetItemId() then
	                    stackz = item.nCount
					    found = true
					    break
	                end
	    
		        end
		    end
		
        elseif category ==107 then --Relics
		    if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Omni-Plasm & Relics"] ~=nil then
		        for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Omni-Plasm & Relics"]) do
		            local itemz2 = tEventArgs.itemNew:GetItemId()
			        if itemz2 == item.itemMaterial:GetItemId() then
	                    stackz = item.nCount
					    found = true
					    break
	                end
	            end
	        end

		elseif category ==109 then --Crafting Supplies
		    if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Miscellaneous"] ~=nil then
		        for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Miscellaneous"]) do
		            local itemz2 = tEventArgs.itemNew:GetItemId()
			        if itemz2 == item.itemMaterial:GetItemId() then
	                    stackz = item.nCount
					    found = true
					    break
	                end
	            end
	        end
			if not found then
			    if GameLib.GetPlayerUnit():GetSupplySatchelItems()["Tinctures"] ~=nil then
		            for  _, item in ipairs(GameLib.GetPlayerUnit():GetSupplySatchelItems()["Tinctures"]) do
		                local itemz2 = tEventArgs.itemNew:GetItemId()
			            if itemz2 == item.itemMaterial:GetItemId() then
	                       stackz = item.nCount
					       found = true
					       break
	                   end
	                end
		        end
	      	
	        end
		end
		for _, item in ipairs(GameLib.GetPlayerUnit():GetInventoryItems()) do
		     local itemz2 = tEventArgs.itemNew:GetItemId()
			if itemz2 == item.itemInBag:GetItemId() then
	            stackz = stackz + item.itemInBag:GetStackCount()
			end
		   
        end
	end	
		
      local tNewEntry =
		{
			eType = knType_Item,
			itemInstance = tEventArgs.itemNew,
			nCount = tEventArgs.nCount,
			money = nil,
			fTimeAdded = GameLib.GetGameTime(),
			stack = stackz
		}
		table.insert(self.tQueuedEntryData, tNewEntry)
		if not self.bPause then
			self.timerUpdate:Start()
		end
		
	end
end

-----------------------------------------------------------------------------------------------
-- CASH FUNCTIONS
-----------------------------------------------------------------------------------------------

function ResourceCounter:OnLootStack_CashTimer()
	self.wndCashComplex:Show(false)
	self.wndCashDisplay:SetAmount(0)
end

-----------------------------------------------------------------------------------------------
-- ITEM FUNCTIONS
-----------------------------------------------------------------------------------------------

function ResourceCounter:OnLootStackUpdate(strVar, nValue)
	if self.wndLootStack == nil then
		return
	end
	
	local bCanMove = self.wndLootStack:IsStyleOn("Moveable")
	if bCanMove ~= self.bIsMoveable then
		self.bIsMoveable = bCanMove
		self.wndLootStack:SetStyle("IgnoreMouse", not self.bIsMoveable)
		self.wndLootStack:FindChild("Backer"):Show(self.bIsMoveable)
	end
	
	local fCurrTime = GameLib.GetGameTime()

	-- remove any old items
	for idx, tEntryData in ipairs(self.tEntryData) do   --TODO: time the remove to delay
		if fCurrTime - tEntryData.fTimeAdded >= kfMaxItemTime then
			self:RemoveItem(idx)
		end
	end

	-- add a new item if its time
	if #self.tQueuedEntryData > 0 then
		if fCurrTime - self.fLastTimeAdded >= kfTimeBetweenItems then
			self:AddQueuedItem()
		end
	end

	-- update all the items
	self:UpdateDisplay()
	
	if #self.tEntryData == 0 then
		self.timerUpdate:Stop()
	end
end

function ResourceCounter:AddQueuedItem()

	-- gather our entryData we need
	local tQueuedData = self.tQueuedEntryData[1]
	table.remove(self.tQueuedEntryData, 1)
	if tQueuedData == nil then
		return
	end

	if tQueuedData.eType == knType_Item and tQueuedData.nCount == 0 then
		return
	end

	-- ensure there's room
	while #self.tEntryData >= knMaxEntryData do
		if not self:RemoveItem(1) then
			break
		end
	end

	if tQueuedData.itemInstance and tQueuedData.itemInstance:CanTakeFromSupplySatchel() then
		Event_FireGenericEvent("LootStackItemSentToTradeskillBag", tQueuedData)
	end

	-- push this item on the end of the table
	local fCurrTime = GameLib.GetGameTime()
	local nBtnIdx = #self.tEntryData + 1
	self.tEntryData[nBtnIdx] = tQueuedData
	self.tEntryData[nBtnIdx].fTimeAdded = fCurrTime -- adds a delay for vaccuum looting by switching logged to "shown" time

	self.fLastTimeAdded = fCurrTime
end

function ResourceCounter:RemoveItem(idx)
	-- validate our inputs
	if idx < 1 or idx > #self.tEntryData then
		return false
	end

	-- remove that item and alert inventory
	table.remove(self.tEntryData, idx)
	return true
end

function ResourceCounter:UpdateDisplay()
	-- iterate over our entry data updating all the buttons
	for idx, wndEntry in ipairs(self.arEntries) do
		local tCurrEntryData = self.tEntryData[idx]
		local itemCurr = tCurrEntryData and tCurrEntryData.itemInstance or false

		if tCurrEntryData then
			wndEntry:Invoke()
		else
			wndEntry:Show(false, true)
		end
		
		if tCurrEntryData and itemCurr and tCurrEntryData.nButton ~= idx then
			local bGivenQuest = itemCurr:GetGivenQuest()
			local eItemQuality = itemCurr and itemCurr:GetItemQuality() or 1
			wndEntry:SetData(itemCurr)
			wndEntry:SetTooltipDoc(nil)
			wndEntry:FindChild("Text"):SetTextColor(bGivenQuest and "white" or karEvalColors[eItemQuality])
			wndEntry:FindChild("LootIcon"):SetSprite(bGivenQuest and "sprMM_QuestGiver" or itemCurr:GetIcon())
			wndEntry:FindChild("RarityBracket"):SetSprite(bGivenQuest and "sprTooltip_Header_White" or karQualitySquareSprite[eItemQuality])

			if tCurrEntryData.nCount == 1 then
			if  tCurrEntryData.stack >1 then
				   wndEntry:FindChild("Text"):SetText(String_GetWeaselString(Apollo.GetString("CombatLog_MultiItem"), tCurrEntryData.nCount, itemCurr:GetName()).." (".. String_GetWeaselString(tCurrEntryData.stack)..")")
			    else
				wndEntry:FindChild("Text"):SetText(itemCurr:GetName())
				end
			else
			
				   if  tCurrEntryData.stack >1 then
				    wndEntry:FindChild("Text"):SetText(String_GetWeaselString(Apollo.GetString("CombatLog_MultiItem"), tCurrEntryData.nCount, itemCurr:GetName()).." (".. String_GetWeaselString(tCurrEntryData.stack)..")")
			    else
				wndEntry:FindChild("Text"):SetText(String_GetWeaselString(Apollo.GetString("CombatLog_MultiItem"), tCurrEntryData.nCount, itemCurr:GetName()))
				end
			end
			
			tCurrEntryData.nButton = idx
		end
	end

	self.wndLootStack:FindChild("LootFloaters"):ArrangeChildrenVert(Window.CodeEnumArrangeOrigin.RightOrBottom)
end

local ResourceCounterInst = ResourceCounter:new()
ResourceCounterInst:Init()
